package support;

import java.io.*;
 
import org.kxml.*;
import org.kxml.parser.*;
import org.kxml.kdom.*;

/** 
 *  This class parses a DB format XML document into a tree and provides
 *  convenient methods to access the data. It also provides methods to
 *  assembly correct DB format XML segments.
 * 
 *  A DB format XML document emulates the data structure inside a
 *  relational database. It contains a number of data "rows" under the
 *  root element. Each row belongs to a data "table" and the name of that
 *  corresponding table is the tagname of the row element. Each "row" has
 *  several "fields". The field name of a field is its tagname and the
 *  value is the text body it encloses. Please refer to the article for
 *  more details.
 * 
 */
public class DBXMLParser {
  
  private Document doc;

  /**
   *  Parse a new document from a "Reader"
   */
  public DBXMLParser (Reader reader) throws Exception {

    XmlParser parser = new XmlParser (reader);
    doc = new Document();
    doc.parse(parser);

  }

  /**
   *  Parse a new document from an "InputStream"
   */
  public DBXMLParser (InputStream ins) throws Exception {

    InputStreamReader reader = new InputStreamReader(ins);
    XmlParser parser = new XmlParser (reader);
    doc = new Document();
    doc.parse(parser);

  }

  /**
   *  The number of rows in table "tableName"
   */
  public int getRowCount( String tableName ) {
    Element root = doc.getRootElement();
    int result = 0;
    // Iterate through the children of the root node.
    for ( int i = 0 ; i < root.getChildCount(); i++ ) {
     // Only the "ELEMENT" type nodes count.
     if ( root.getType(i) == Xml.ELEMENT ) {
        Element tr = (Element) root.getChild(i);
        if ( tableName.equals(tr.getName()) ) {
          result++;
        }
      }
    }
    return result;
  }

  /**
   *  Find the "index"th data row (element) named "tableName"
   *  and then get the value for field "fieldName".
   */
  public String getFieldValue ( String tableName, String fieldName, 
                                int index ) throws Exception {

    Element root = doc.getRootElement();
    Element row = null, field = null;
    int currentIndex = 0;

    // Iterate through children of root.
    for ( int i=0; i<root.getChildCount(); i++) {
      // Only "ELEMENT" type nodes is processed.
      if ( root.getType(i) == Xml.ELEMENT ) {
        Element tr = (Element) root.getChild(i);
        // Tag name must match tableName. 
        if ( tableName.equals( tr.getName() ) ) {
          // index must match input parameter.
          if ( currentIndex == index ) {
            row = (Element) root.getChild(i);
            break;
          }
          currentIndex++;
        }
      }
    }

    // return null if no match is found
    if ( row == null ) {
      return null;
    }

    // Iterate through children of the matched row.
    for ( int i=0; i<row.getChildCount(); i++) {
      if ( row.getType(i) == Xml.ELEMENT ) {
        Element tf = (Element) row.getChild(i);
        // Match the fieldName to tag name.
        if ( fieldName.equals( tf.getName() ) ) {
          field = tf;
          break;
        }
      }
    }
    
    // return null if no match is found
    if ( field == null ) {
      return null;
    } else {
      return field.getText(0);
    }

  }

  /**
   *  Compose an XML string representing a row of data with
   *  a number of fields.
   */
  public static String rowString (String tableName, String[] fieldNames,
                                  String[] fieldValues) throws Exception {

    StringBuffer sb = new StringBuffer();
    // Start tag of the row.
    sb.append("<"+tableName+">");
    for (int i=0; i<fieldNames.length; i++) {
      // Start tag of a field.
      sb.append("<"+fieldNames[i]+">");
      // If the field contains text, write text as
      // the body of the field tag.
      if ( fieldValues[i] != null ) {
        sb.append(fieldValues[i]);
      }
      // Close the field tag.
      sb.append("</"+fieldNames[i]+">");
    }
    // Close the row tag.
    sb.append("</"+tableName+">");
    
    // Output the composed string.
    return sb.toString();
  }

  /**
   *  export the whole tree to an XML string
   */
  public String toString () {
    return doc.toString();
  }

}
